package org.rcisoft.code.service.impl;

import com.alibaba.druid.pool.DruidDataSource;
import freemarker.template.Configuration;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.TransactionFactory;
import org.apache.ibatis.transaction.jdbc.JdbcTransactionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.rcisoft.code.dao.GeneratorRepository;
import org.rcisoft.code.model.CodeSchema;
import org.rcisoft.code.model.CodeTable;
import org.rcisoft.code.service.CodeService;
import org.rcisoft.common.component.FamilyDbUtils;
import org.rcisoft.common.component.Global;
import org.rcisoft.code.model.DbColumnModel;
import org.rcisoft.code.model.DbTableModel;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultSystemExceptionEnums;
import org.rcisoft.core.service.BaseService;
import org.rcisoft.core.util.DateUtil;
import org.rcisoft.core.util.DbUtil;
import org.rcisoft.core.util.ZipCompress;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * Created by lcy on 17/6/28.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CodeServiceImpl extends BaseService implements CodeService {

    private static final long serialVersionUID = -3827321264317955429L;

    @Autowired
    private FamilyDbUtils familyDbUtils;

    @Autowired
    private Configuration freeMarkerConfiguration;

    @Autowired
    private Global global;



    public List<CodeTable> getCodeTablesBySchema(CodeSchema schema){
        List<CodeTable> tables = new ArrayList<>();
        Connection conn  = familyDbUtils.getConnection(schema);;
        ResultSet rs = null;
        DatabaseMetaData dbmd = null;
        CodeTable ct = null;

        try {
            dbmd = (DatabaseMetaData) conn.getMetaData();
            rs=(ResultSet) dbmd.getTables(null, schema.getDbName(), "%", null);
            while(rs.next()){
                ct = new CodeTable();
                ct.setTableName(rs.getString("TABLE_NAME"));
                tables.add(ct);
            }
        }catch (Exception e){
            log.error(e.getMessage());
        }finally {
            this.killConnection(rs,conn);
        }
        return tables;
    }

    @Override
    public String generateCode(CodeSchema codeSchema) throws Exception{
        String filePath = null;
        /*blank*/
        if(StringUtils.isBlank(codeSchema.getTables()))
            throw new ServiceException(ResultSystemExceptionEnums.PARAMETER_ERROR);
        codeSchema.setMapperLocations(global.getMAPPER_LOCATIONS());
        List<DbTableModel> tableModels = new ArrayList<DbTableModel>();
        List<DbColumnModel> models = null;
        /*tables*/
        String[] tables = codeSchema.getTables().split(",");
        if(ArrayUtils.isEmpty(tables))
            throw new ServiceException(ResultSystemExceptionEnums.PARAMETER_ERROR);
        DbTableModel tableModel = null;
        DbColumnModel model = null;
        SqlSession session = null;
        /*表描述*/
        try {
            session = DbUtil.getMybatisSqlSessionByCodeSchema(codeSchema);
            //获取对象
            GeneratorRepository dao = session.getMapper(GeneratorRepository.class);
            if(null == dao)
                throw new ServiceException(ResultSystemExceptionEnums.ERROR);
            for(String table : tables) {
                tableModel = new DbTableModel();
                // table query
                Map<String, String> result = dao.queryTable(table.toLowerCase());
                String tableName = (null == result.get(DbTableModel.CONS_TABLE_NAME)? "" : result.get(DbTableModel.CONS_TABLE_NAME).toLowerCase());
                tableModel.setTableName(tableName);
                tableModel.setTableRemark(result.get(DbTableModel.CONS_TABLE_COMMENT));
                models = new ArrayList<DbColumnModel>();
                List<Map<String, String>> cols = dao.queryColumns(tableName);
                //column
                if(CollectionUtils.isEmpty(cols))
                    continue;
                boolean flag = false;
                for(Map<String,String> col : cols){
                    model = new DbColumnModel();
                    /* 根据 CREATE_BY 判断是否存在7个字段 */
                    if("CREATE_BY".equals(col.get(DbColumnModel.CONS_COLUMN_NAME).toUpperCase()))
                        flag = true;
                    model.copyColumnFromSqlMap(col);
                    models.add(model);
                }
                tableModel.setIdEntity(flag);
                //table -> column
                tableModel.setColumnModels(models);
                tableModels.add(tableModel);
            }
            filePath = this.productCodeFromTable(codeSchema.getAuthor(),codeSchema.getBasePackage(),tableModels);
        }catch (Exception e){
            log.error(e.getMessage());
            throw new ServiceException(ResultSystemExceptionEnums.ERROR);
        }finally {
            session.close();
        }
        return filePath;
    }

    /**
     * 通过freemarker 成功多层代码
     * @param tableModels
     * @return
     */
    private String productCodeFromTable(String author,String packageName,List<DbTableModel> tableModels) {

        Map root = null;
        List<File> fileList = new ArrayList<File>();
        File file = null;
        String dirPath = global.getBaseUploadLocation()
                + global.getCodeGenerateLocation() +
                File.separator + DateUtil.getSimepleDate("yyyyMMddHHmmss", new Date());
        String zipPath = dirPath + ".zip";
        String zipDirPath = dirPath;
        //将. 换/
        packageName = StringUtils.isBlank(packageName)? "org.rcisoft.business" : packageName;
        String pathPattern = packageName.replaceAll("\\.","\\" +File.separator);
        //存在删除
        dirPath = dirPath + File.separator + pathPattern;  //!! 此行一定要在 zipPath 生成后
        File dirF = new File(dirPath);
        File zipFile = new File(zipPath);
        dirF.deleteOnExit();
        zipFile.deleteOnExit();
        //生成
        try {
            for(DbTableModel dbTableModel : tableModels) {
                root = new HashMap<String,Object>();
                String modelNameUpperCamel = dbTableModel.getEntityName();
                String basePackage = packageName +"."+modelNameUpperCamel.toLowerCase();
                root.put("author", author);
                root.put("table", dbTableModel);
                root.put("basePackage", basePackage);
                //通过一个文件输出流，就可以写到相应的文件中
                //1.entity , mapper
                String newPath = dirPath+File.separator+modelNameUpperCamel.toLowerCase();
                file = this.generateEntityModel(newPath,dbTableModel,root,"Entity");
                fileList.add(file);

                file = this.generateEntityModel(newPath,dbTableModel,root,"Mapper");
                fileList.add(file);
                //2.service
                file = this.generateEntityModel(newPath,dbTableModel,root,"Service");
                fileList.add(file);
                //3.serviceImpl
                file = this.generateEntityModel(newPath,dbTableModel,root,"ServiceImpl");
                fileList.add(file);
                //4.dao
                file = this.generateEntityModel(newPath,dbTableModel,root,"Repository");
                fileList.add(file);
                //5.controller
                file = this.generateEntityModel(newPath,dbTableModel,root,"Controller");
                fileList.add(file);
            }
            ZipCompress.compressExe(zipDirPath,zipPath);
        } catch (Exception e) {
            zipPath = null;
            e.printStackTrace();
        }
        return zipPath;
    }

    /**
     * 关闭 connection
     * @param rs
     * @param conn
     */
    private void killConnection(ResultSet rs,Connection conn){
        try {
            if(null != rs)
                rs.close();
            if(null != conn)
                conn.close();
        } catch (SQLException e) {
            log.error(e.getMessage());
        }
    }

    /**
     * model  构建  entity 层
     * @param dirPath
     * @param dbTableModel
     * @param root
     * @return
     */
    private File generateEntityModel(String dirPath,DbTableModel dbTableModel,Map root,String suffix){
        //源文件夹
        File dirFile = null;
        //目标文件
        File file = null;
        FileWriter out = null;
        Template temp = null;
        if(suffix.equals("Repository"))
            dirFile = new File(dirPath + File.separator + "dao");
        else if(suffix.equals("ServiceImpl"))
            dirFile = new File(dirPath + File.separator + "service"+File.separator+"impl");
        else
            dirFile = new File(dirPath + File.separator + suffix.toLowerCase());
        //不存在 创建文件夹
        if(!dirFile.exists())
            dirFile.mkdirs();
        if(suffix.equals("Repository"))
            file = new File(dirPath  + File.separator +  "dao" +
                    File.separator +   dbTableModel.getEntityName() + suffix+".java");
        else if(suffix.equals("ServiceImpl"))
            file = new File(dirPath  + File.separator +  "service"+File.separator+"impl" +
                    File.separator +   dbTableModel.getEntityName() +suffix+".java");
        else if(suffix.equals("Entity"))
            file = new File(dirPath  + File.separator +  suffix.toLowerCase() +
                    File.separator +   dbTableModel.getEntityName() +".java");
        else if(suffix.equals("Mapper"))
            file = new File(dirPath  + File.separator +  suffix.toLowerCase() +
                    File.separator +   dbTableModel.getEntityName() +suffix+".xml");
        else
            file = new File(dirPath  + File.separator +  suffix.toLowerCase() +
                    File.separator +   dbTableModel.getEntityName() +suffix+".java");
        try {
            out = new FileWriter(file);
            //fileList.add(file);
            if(suffix.equals("Entity") && !dbTableModel.isIdEntity())
                temp = freeMarkerConfiguration.getTemplate("codeftl" + File.separator + suffix.toLowerCase() + "NotCommon" +".ftl");
            else
                temp = freeMarkerConfiguration.getTemplate("codeftl" + File.separator + suffix.toLowerCase()+".ftl");
            temp.process(root, out);
        }catch (Exception e){
            log.error(e.getMessage());
        }finally {
            try {
                if(out!=null)
                    out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        return file;
    }

}
